@@ -20,7 +20,7 @@
 extern int verbose;
 extern int prefer_ata12;
 
-static const int default_timeout_secs = 15;
+static const unsigned int default_timeout_secs = 15;
 
 /*
  * Taskfile layout for SG_ATA_16 cdb:
